#include "Subdivide.h"
#include <iostream>
#include <fstream>
#include "util.h"

using namespace std;

Subdivide::Subdivide(Mesh * m_in) {
   mesh = m_in;
   int i = 0;

   Mesh::FaceIterator fIter = mesh->faceIterator();
   for (; !fIter.end(); fIter++) {
      for (Face::EdgeAroundIterator around = fIter.face()->iterator(); !around.end(); around++) {
	 around.edge()->subdPointID = i++;
      }	
   }
}


void Subdivide::doMask1() {
   cout << "Doing subdivision with mask 1 (Doo-Sabin variant)" << endl;
}

void Subdivide::doMask2() {
   cout << "Doing subdivision with mask 2 (Catmull-Clark variant)" << endl;
}

void Subdivide::doMask3() {
   cout << "Doing subdivision with mask 3" << endl;
}

void Subdivide::doMask4() {
   cout << "Doing subdivision with mask 4" << endl;
}

void Subdivide::outputSubdSurface(const char * obj_file) {
   ofstream out(obj_file);
   check_error (!out, "Cannot find output file.");

   Mesh::FaceIterator fIter = mesh->faceIterator();
   for (; !fIter.end(); fIter++) {
      for (Face::EdgeAroundIterator around = fIter.face()->iterator(); !around.end(); around++) {
	 Vector3 sp = around.edge()->subdPoint;
	 out << "v " << sp.x() << " " << sp.y() << " " << sp.z() << endl;
      }
   }
 
   for (fIter.reset(); !fIter.end(); fIter++) {
      Face::EdgeAroundIterator around = fIter.face()->iterator();
      out << "f";
      for ( ; !around.end(); around++)
	 out << " " << (around.edge()->subdPointID + 1);
      out << endl;
   }

   Mesh::EdgeIterator eIter = mesh->edgeIterator();
   for (; !eIter.end(); eIter++) {
      Edge * e = eIter.edge();
      if (!e->isBoundary()) {
	 out << "f " << (e->subdPointID + 1) << " " << (e->pair->next->subdPointID + 1)
	      << " " << (e->pair->subdPointID + 1) << " " << (e->next->subdPointID + 1) << endl;
      }
   }

   Mesh::VertexIterator vIter = mesh->vertexIterator();
   for (; !vIter.end(); vIter++) {
      if (!vIter.vertex()->isBoundary()) {
	 Vertex::EdgeAroundIterator around = vIter.vertex()->iterator();
	 out << "f";
	 for ( ; !around.end(); around--)
	    out << " " << (around.edge_out()->subdPointID + 1);
	 out << endl;
      }
   }

   out.close();
}

