/** \file util.h
File contains various utility functions.
*/
#ifndef UTIL_CM
#define UTIL_CM
#include <iostream>
#include <string>

static void check_error (bool expr, const std::string & message);
static void check_warn (bool expr, const std::string & message);
static void usage();

/** If expression \a expr is true, \a message is output and program terminates.
\param expr: Exression to check
\param message: Error message to output.
*/
static void check_error (bool expr, const std::string & message) {
   if (expr) {
      std::cerr << "Error: " << message << std::endl;
      exit(1);
   }
}

/** If expression \a expr is true, warning \a message is output.
\param expr: Exression to check
\param message: Warning message to output.
*/
static void check_warn (bool expr, const std::string & message) {
   if (expr)
      std::cerr << "Warning: " << message << std::endl;
}

//static void MSKAPI printMosekStr(void *handle, char str[]) {
//   std::ostream * mosekOut = (std::ostream *) handle;
//   check_error (!mosekOut, "No output stream specified for MOSEK.");
//   *mosekOut << str; 
//} 

/** 
Usage printout function.
*/
static void usage() {
   std::cerr << "USAGE:\ncs175_Subd.exe <REQUIRED OPTIONS> "
      << " [OPTIONAL OPTIONS] | <INPUT_FILE_WITH_OPTIONS>\n" << std::endl;
   std::cerr << "OPTIONS:\n"
      << "  REQUIRED OPTIONS:\n"
      << "    -io <input obj file>\n"
      << "    -oo <output obj file>\n"
      << "    <text file with options>\n"
      << "  OPTIONAL OPTIONS:\n"
      << "    -n <num_steps> (default is 1)\n"
      << "    -m <variant_of_weights> (default is 1)\n"
      << std::endl;

   exit(1);
}

/** 
Structure to store command line options
input by user
*/
struct ProgramOptions {
   /** Input (OBJ or CON) file */
   std::string file_in;

   /** Output file (CON format). */
   std::string obj_file_out;

   /** True if OBJ input file specified. */
   bool read_obj;

   /** True if OBJ file needs to be written. */
   bool write_obj;

   /** How many subdivisions to perform */
   int num_steps;

   /** Subdivision variant */
   int method;

   ProgramOptions(): file_in(""), obj_file_out(""),
      read_obj(false), write_obj(false),
      num_steps(1), method(1)
   {}
};

#endif

